'use strict';

async function fetchImage(url, callback, headers) {
  let _headers = {};

  if (Object.keys(headers).length) {
    Object.entries(headers).forEach(([h, v]) => {
      _headers[h] = v;
    });
  }
  const f = await fetch(url, {
    method: "GET",
    headers: _headers
  });
  const blob = await f.blob();
  callback(blob);
}

L.TileLayer.BetterWMS = L.TileLayer.WMS.extend({
  initialize: function (url, options, headers) {
    L.TileLayer.WMS.prototype.initialize.call(this, url, options);
    this.headers = { ...headers.headers };
  },
  createTile(coords, done) {
    const url = this.getTileUrl(coords);
    const img = document.createElement("img");
    img.setAttribute("role", "presentation");

    fetchImage(
      url,
      resp => {
        const reader = new FileReader();
        reader.onload = () => {
          img.src = reader.result;
        };
        reader.readAsDataURL(resp);
        done(null, img);
      },
      { ...this.headers }
    );
    return img;
  },

  onAdd: function (map) {
    // Triggered when the layer is added to a map.
    //   Register a click listener, then do all the upstream WMS things
    L.TileLayer.WMS.prototype.onAdd.call(this, map);
    map.on('click', this.getFeatureInfo, this);
  },

  onRemove: function (map) {
    // Triggered when the layer is removed from a map.
    //   Unregister a click listener, then do all the upstream WMS things
    L.TileLayer.WMS.prototype.onRemove.call(this, map);
    map.off('click', this.getFeatureInfo, this);
  },

  getFeatureInfo: function (evt) {
    var url = this.getFeatureInfoUrl(evt.latlng),
      showResults = L.Util.bind(this.showGetFeatureInfo, this);
    fetch(url)
      .then((response) => {
        return response.json();
      })
      .then((data) => {
        var err = typeof data === 'object' ? null : data;
        showResults(err, evt.latlng, data);
      })
      .catch((error) => {
        showResults(error);
      });
  },

  getFeatureInfoUrl: function (latlng) {
    // Construct a GetFeatureInfo request URL given a point
    var point = this._map.latLngToContainerPoint(latlng, this._map.getZoom()),
      size = this._map.getSize(),

      params = {
        request: 'GetFeatureInfo',
        service: 'WMS',
        srs: 'EPSG:4326',
        styles: this.wmsParams.styles,
        transparent: this.wmsParams.transparent,
        version: this.wmsParams.version,
        format: this.wmsParams.format,
        bbox: this._map.getBounds().toBBoxString(),
        height: size.y,
        width: size.x,
        layers: this.wmsParams.layers,
        query_layers: this.wmsParams.layers,
        info_format: 'application/json'
      };

    params[params.version === '1.3.0' ? 'i' : 'x'] = point.x;
    params[params.version === '1.3.0' ? 'j' : 'y'] = point.y;

    return this._url + L.Util.getParamString(params, this._url, true);
  },

  buildInfoTable: function (data) {
    if(data.features.length) {
        let table = document.createElement('table');
        table.classList = 'attributes-table';
        let tableHeader = document.createElement('tr');
        tableHeader.classList = 'attribute-header';
        let tableHeaderAttribute = document.createElement('th');
        let tableHeaderValue = document.createElement('th');
        tableHeaderAttribute.innerHTML = 'Atributo';
        tableHeaderValue.innerHTML = 'Valor';
        tableHeader.appendChild(tableHeaderAttribute);
        tableHeader.appendChild(tableHeaderValue);
        table.appendChild(tableHeader);
        Object.entries(data.features[0].properties).forEach(([k,v]) => {
            let tableRow = document.createElement('tr');
            let tableEntryKey = document.createElement('td');
            let tableEntryValue = document.createElement('td');
            tableEntryKey.innerHTML = k;
            tableEntryKey.classList = 'attribute-entry';
            tableRow.classList = 'attribute-row';
            tableEntryValue.innerHTML = v;
            tableRow.appendChild(tableEntryKey);
            tableRow.appendChild(tableEntryValue);
            table.appendChild(tableRow);
        });
        
        return table;
    } else {
        let message = document.createElement('p');
        message.innerHTML = 'No hay información disponible';
        return message;
    }
},

  showGetFeatureInfo: function (err, latlng, content) {
    if (err) { console.log(err); return; } // do nothing if there's an error

    // Otherwise show the content in a popup, or something.
    L.popup({ maxWidth: 800 })
      .setLatLng(latlng)
      .setContent(this.buildInfoTable(content))
      .openOn(this._map);
  }
});

L.tileLayer.betterWms = function (url, options, headers) {
  return new L.TileLayer.BetterWMS(url, options, headers);
};